$alias fdopen='fdopen'(%val,%ref)
      include '/usr/include/phigs.f1.h'
      program sampleval      ! program "sampleval.f" 
C This program demonstrates the use of a requested choice device in
C combination with a set of sampled valuators
C The choice value is used to indicate which set of three values to
C sample
C This program goes through the following steps
C 1. Open PHIGS
C 2. Open a workstation which is a double buffer X window named phigs_window 
C    with input capabilities
C 3.Prompts the user to set the knobs and press a mouse button (choice device)
C 4. Samples the indicated valuator devices
C 5. The program will terminate if the user inputs a break key
C
      include '/usr/include/phigs.f2.h'
      integer*4   BufferLength   ! currently unused 
      integer*4   WorkID         ! workstation id 
      integer*4   ConnectID      ! communication channel id 
      integer*4   WorkType       ! workstation type 
      integer*4   SpecType       ! specific workstation type
      integer*4   Unittype       ! device unit type
      integer*4   Choice         ! Choice from choice device
      integer*4   ChoiceStatus   ! Choice Status from device
      integer*4   Pointer        ! Pointer device number 
	integer*4   ERRIND           ! error indicator
      real   EchoVolume(6)       ! Echo Volume Array
      real   DX,DY,DZ            ! Display Size in DCs
      real   RX,RY,RZ            ! Display Size in rasters
      logical   Done             ! loop control variable
      parameter (BufferLength=0, WorkID=1)
      parameter (WorkType=POIDDX)
      parameter (Pointer=1)
	integer*4 xmin, xmax, ymin, ymax, zmin, zmax
      parameter (xmin=1,xmax=2,ymin=3,ymax=4,zmin=5,zmax=6)
C Variables for packing data record
C Integer part of data record
      integer*4       IntCount   ! DATREC Integer count
      parameter(IntCount=2)      ! DATREC Integer count value
      integer*4       Ints(IntCount)   ! DATREC Integer array
C Real part of data record
      integer*4       RealCount  ! DATREC Real count
      parameter(RealCount=2)     ! DATREC Real count value
      real            Reals(RealCount)   ! DATREC Real array
C String part of data record
      integer*4   StrCount       ! DATREC String count
      parameter(StrCount=0)      ! DATREC String count value
      integer*4   StrLength      ! DATREC String length
      parameter(StrLength=1)     ! DATREC String length value
      character*1   Strings(1)   ! DATREC String array
C Returned data record
      integer*4       RecCount   ! DATREC Element count
      parameter(RecCount=1)      ! DATREC Element count value
      integer*4       Length     ! DATREC Element return length
      character*80  DATREC(RecCount)   ! DATREC array
      integer*4 fdopen				! to get file descriptor

      call popph(fdopen(fnum(7), 'w'//char(0)), 0)	! open phigs
      call pue004("/dev/screen/phigs_window", ConnectID)
      call popwk(WorkID, ConnectID, WorkType)   ! open the workstation 

C Get the specific workstation type and range of device coordinates
      call pqwkc(WorkID, ERRIND, ConnectID, SpecType)
        call pqdsp3(SpecType,ERRIND,Unittype,DX,DY,DZ,RX,RY,RZ)
C Set echo limits using returned device coordinates
      EchoVolume(xmin)=DX*0.2
      EchoVolume(xmax)=DX*0.8
      EchoVolume(ymin)=DY*0.2
      EchoVolume(ymax)=DY*0.4
      EchoVolume(zmin)=DZ*0.0
      EchoVolume(zmax)=DZ*1.0
C Pack echo colours and min and max return values into data record
      Ints(1)=2   ! slide text colour
      Reals(1)=0.0
      Reals(2)=10.0
      call pprec(IntCount,Ints,
     +             RealCount,Reals,
     +             StrCount,StrLength,Strings,
     +             RecCount,
     +             ERRIND,
C Returned Length and data record
     +             Length,DATREC)
C Initalize the valuator 10
      call pinvl3(WorkID,
     +            10,  ! Valuator Device Number
     +            5.0, ! Initial valuator value
     +            1,   ! Prompt/Echo Type - Only value 1 is available
     +            EchoVolume, ! Size of echo volume
     +            Length,  ! Length of data record
     +            DATREC)  ! Data record
      EchoVolume(xmin)=DX*0.2
      EchoVolume(xmax)=DX*0.8
      EchoVolume(ymin)=DY*0.6
      EchoVolume(ymax)=DY*0.8
      EchoVolume(zmin)=DZ*0.0
      EchoVolume(zmax)=DZ*1.0
C Pack echo colours and min and max return values into data record
      Ints(1)=3   ! slide text colour
      Reals(1)=-10.0
      Reals(2)=0.0
      call pprec(IntCount,Ints,
     +             RealCount,Reals,
     +             StrCount,StrLength,Strings,
     +             RecCount,
     +             ERRIND,
C Returned Length and data record
     +             Length,DATREC)
C Initalize the valuator 11
      call pinvl3(WorkID,
     +            11,   ! Input Device Number
     +            -5.0, ! Initial valuator value
     +            1,    ! Prompt/Echo Type - Only value 1 is available
     +            EchoVolume, ! Size of echo volume
     +            Length,  ! Length of data record
     +            DATREC)  ! Data record

C Put all knobs on the knob box into sample mode.
C   Bottom row left to right
      call psvlm(WorkID,10,PSAMPL,PECHO)
      call psvlm(WorkID,11,PSAMPL,PECHO) 
      call psvlm(WorkID,12,PSAMPL,PNECHO) ! No Echo
C   Middle row left to right
      call psvlm(WorkID,13,PSAMPL,PNECHO) ! No Echo
      call psvlm(WorkID,14,PSAMPL,PNECHO) ! No Echo
      call psvlm(WorkID,15,PSAMPL,PNECHO) ! No Echo
C   Top row left to right
      call psvlm(WorkID,16,PSAMPL,PNECHO) ! No Echo
      call psvlm(WorkID,17,PSAMPL,PNECHO) ! No Echo
      call psvlm(WorkID,18,PSAMPL,PNECHO) ! No Echo

      Done=.false.
      do while (.not. Done)
         print *,"Set the knobs and press a mouse button."
         call prqch(WorkID,Pointer,ChoiceStatus,Choice) ! request a choice
         if (ChoiceStatus .eq. POK) then ! sample the valuators
            if (Choice .eq. 1) then
               print *,"Bottom Knob Values"
	         call sampleprint(WorkID,10,11,12) ! bottom
             else if (Choice .eq. 2) then
		         print *,"Middle Knob Values"
		         call sampleprint(WorkID,13,14,15) ! middle
                  else if (Choice .eq. 3)  then
		              print *,"Top Knob Values"
	                    call sampleprint(WorkID,16,17,18) ! top
             endif
          else 
	       Done=.true. 
             print *,"ChoiceStatus: ",ChoiceStatus ! print error status
          endif
      end do

      call pclwk(WorkID)      ! close the workstation 
      call pclph              ! close PHIGS 
      stop                    ! stop processing 
      end                     ! end of main program 
C**********************************************************************
C This routine samples the indicated devices and prints the input value
C**********************************************************************
      subroutine sampleprint(WorkID,Left,Middle,Right)
      integer*4 WorkID
      integer*4 Left,Middle,Right
      include '/usr/include/phigs.f2.h'

      real Value 
      call psmvl(WorkID,Left,Value)   ! Sample value for left knob
      print *,"Left  : ",Value
      call psmvl(WorkID,Middle,Value) ! Sample value for middle knob
      print *,"Middle: ",Value
      call psmvl(WorkID,Right,Value) ! Sample value for right knob
      print *,"Right : ",Value
      return
      end

